using System;
using System.Collections.Generic;
using DataContracts;

namespace Ato.EN.IntegrationServices.CodeGenerationFITR
{
    public partial class FITR2019Validator
    {

        #region Cross Form Rules - not supported yet

        #region VR.ATO.FITR.434034

        /*  VR.ATO.FITR.434034
                    If Net capital gain is more than $10,000, a CGT schedule must be present

                Legacy Rule Format:
                    IF [FITR23] > 10000 AND  (COUNT(SCHEDULE = "CGTS") = 0)  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR23] = FITR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    ^FITR23 > 10000 AND  (CountDocument('CGTS') = 0)

        Data Elements:
            
                    RP:^FITR23 = tns:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOFITR434034(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434034

            assertion = false; // = (report.FITR23.GetValueOrDefault() > 10000 && @"CGTS") == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434034",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"If Net capital gain is more than $10,000, a CGT schedule must be present",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434034" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR23", Value = report.FITR23.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434034

        #region VR.ATO.FITR.434036

        /*  VR.ATO.FITR.434036
                    Net capital gains in return must equal net capital gain in CGT schedule.

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND [FITR23] <> [CGTS131]
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS131] = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [FITR23] = FITR:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND ^FITR23 <> ^CGTS131

        Data Elements:
            
                    RP:^FITR23 = tns:Income.CapitalGainsNet.Amount
            
                    RP:^CGTS131 = CGTS:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    */
        protected void VRATOFITR434036(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434036

            assertion = false; // = (@"CGTS") == 1 && report.FITR23.GetValueOrDefault() != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434036",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital gains in return must equal total net capital gain in CGT schedule.",
                    //Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434036" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR23", Value = report.FITR23.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434036

        #region VR.ATO.FITR.434045

        /*  VR.ATO.FITR.434045
                    Tax withheld amount must not be less than the sum of amounts present in the schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND [FITR32] > 0 AND (([FITR72] + 1) < SUM([PSS23]))  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR32] = FITR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [FITR72] = FITR:RP:rvctc2.02.02:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount
                    [PSS23] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND ^FITR32 > 0 AND ((^FITR72 + 1) < Sum(^PSS23))

        Data Elements:
            
                    RP:^FITR72 = tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount
            
                    RP:^FITR32 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS23 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                    */
        protected void VRATOFITR434045(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434045

            assertion = false; // = (@"PSS") == 1 && report.FITR32.GetValueOrDefault() > 0 && report.FITR72.GetValueOrDefault() + 1 < ( == null ? 0 : .Sum(f => f.)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434045",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld amount must not be less than the sum of amounts present in the schedule",
                    LongDescription = @"The 'Credit for tax withheld where ABN/TFN not quoted' amount must equal the sum of the Tax withheld amounts on the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldTFNNotQuotedAndABNNotQuoted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434045" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR32", Value = report.FITR32.GetValueOrDefault().ToString() });

                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR72", Value = report.FITR72.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434045

        #region VR.ATO.FITR.434046

        /*  VR.ATO.FITR.434046
                    Gross payment amount must equal the sum of amounts in the Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS19]) <> [FITR32]  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR32] = FITR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    [PSS19] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND Sum(^PSS19) <> ^FITR32

        Data Elements:
            
                    RP:^FITR32 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
            
                    RP:^PSS19 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        protected void VRATOFITR434046(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434046

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) != report.FITR32.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross payment amount must equal the sum of amounts in the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434046" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR32", Value = report.FITR32.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434046

        #region VR.ATO.FITR.434056

        /*  VR.ATO.FITR.434056
                    Other income must not be less than Gross payment amounts on Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS18]) > [FITR47]   
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR47] = FITR:RP:bafpr1.02.04:Income.Other.Amount
                    [PSS18] = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND Sum(^PSS18) > ^FITR47

        Data Elements:
            
                    RP:^FITR47 = tns:Income.Other.Amount
            
                    RP:^PSS18 = PSS:RP.Payer.{PSSeqNum}:lrla.02.00:Remuneration.PaymentToForeignResidentGross.Amount
                    */
        protected void VRATOFITR434056(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434056

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) > report.FITR47.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434056",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Other income must not be less than Gross payment amounts on Payment Summary schedule",
                    LongDescription = @"The 'Other income' amount must not be less than the sum of all 'Gross payment' amounts on the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:Income.Other.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434056" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR47", Value = report.FITR47.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434056

        #region VR.ATO.FITR.434080

        /*  VR.ATO.FITR.434080
                    Credit: FRW (excluding capital gains) must not be less than the total of all FRW amounts in Payment Summary schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "PSS") = 1) AND SUM([PSS22]) > ([FITR71] + 1)  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR71] = FITR:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                    [PSS22] = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount

            Technical Business Rule Format:
                    (CountDocument('PSS') = 1) AND Sum(^PSS22) > (^FITR71 + 1)

        Data Elements:
            
                    RP:^FITR71 = tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            
                    RP:^PSS22 = PSS:RP.Payer.{PSSeqNum}:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                    */
        protected void VRATOFITR434080(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434080

            assertion = false; // = (@"PSS") == 1 && ( == null ? 0 : .Sum(f => f.)) > report.FITR71.GetValueOrDefault() + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit: FRW (excluding capital gains) must not be less than the total of all FRW amounts in Payment Summary schedule",
                    LongDescription = @"The 'Credit: foreign resident withholding (excluding capital gains)' amount on the main form must not be less than the sum of all 'Foreign resident withholding' amounts on the Payment Summary schedule",
                    //Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434080" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR71", Value = report.FITR71.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434080

       

       

        

        #region VR.ATO.FITR.434088

        /*  VR.ATO.FITR.434088
                    Net capital losses carried forward to later income years must equal amount in CGT schedule

                Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "CGTS") = 1) AND  [FITR207] <> ([CGTS124]+[CGTS125])
                    RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS124] = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [CGTS125] = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    [FITR207] = FITR:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
                    (CountDocument('CGTS') = 1) AND  ^FITR207 <> (^CGTS124+^CGTS125)

        Data Elements:
            
                    RP.Closing(Instant):^FITR207 = tns:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS124 = CGTS:RP.Collectables.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
            
                    RP.Closing(Instant):^CGTS125 = CGTS:RP.CGTOther.Unapplied(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount
                    */
        protected void VRATOFITR434088(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.FITR.434088

            assertion = false; // = (@"CGTS") == 1 && report.FITR207.GetValueOrDefault() !=  + );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.FITR.434289",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Net capital losses carried forward to later income years must equal amount in CGT schedule",
                    LongDescription = @"Net capital losses carried forward to later income years' on the main form must equal the total of the 'Unapplied net capital losses carried forward'  in the Capital Gains Tax schedule",
                    //Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434088" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR207", Value = report.FITR207.GetValueOrDefault().ToString() });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FITR.434088

        #region VR.ATO.FITR.434300

        /*  VR.ATO.FITR.434300
                    Payment summary schedule must be present when Payments where ABN not quoted amount is present

                Legacy Rule Format:
                    IF [FITR32] > 0 AND  (COUNT(SCHEDULE = "PSS") = 0)  
                        RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [FITR32] = FITR:RP:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
                    ^FITR32 > 0 AND  (CountDocument('PSS') = 0)

        Data Elements:
            
                    RP:^FITR32 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                    */
        //protected void VRATOFITR434300(List<ProcessMessageDocument> response, FITR2019 report)
        //{
        //    ProcessMessageDocument processMessage;
        //    bool assertion;

        //    //TODO: Write manual rule for VR.ATO.FITR.434300

        //    assertion = false; // = (report.FITR32.GetValueOrDefault() > 0 && @"PSS") == 0);
        //    if (assertion)
        //    {
        //        processMessage = new ProcessMessageDocument()
        //        {
        //            Code = "CMN.ATO.GEN.434044",
        //            Severity = ProcessMessageSeverity.Error,
        //            Description = @"Payment summary schedule must be present when Payments where ABN not quoted amount is present",
        //            //Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount",
        //            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.FITR.434300" } },
        //        };

        //        /*  Parameters are commented out or non-existant for manual rules 
        //            processMessage.Parameters.Add(new ProcessMessageParameter
        //                { Name = "FITR32", Value = report.FITR32.GetValueOrDefault().ToString() });


        //        */
        //        response.Add(processMessage);
        //    }
        //}
        #endregion // VR.ATO.FITR.434300

        #region VR.ATO.GEN.438022

        /*  VR.ATO.GEN.438022
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS127] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount

            Technical Business Rule Format:
                    (^FITR204 = 'A - Small business active asset reduction') AND (CountDocument('CGTS') = 1) AND ^CGTS127 = NULL

        Data Elements:
            
                    RP:^FITR204 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS127 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                    */
        protected void VRATOGEN438022(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438022

            assertion = false; // = (FITR204 == @"A - Small business active asset reduction" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438022",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business active asset reduction amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR204", Value = FITR204 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438022

        #region VR.ATO.GEN.438024

        /*  VR.ATO.GEN.438024
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS129] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount

            Technical Business Rule Format:
                    (^FITR204 = 'C - Small business roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS129 = NULL

        Data Elements:
            
                    RP:^FITR204 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS129 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                    */
        protected void VRATOGEN438024(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438024

            assertion = false; // = (FITR204 == @"C - Small business roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438024",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR204", Value = FITR204 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438024

        #region VR.ATO.GEN.438025

        /*  VR.ATO.GEN.438025
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS138] = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount

            Technical Business Rule Format:
                    (^FITR204 = 'D - Small business 15 year exemption') AND (CountDocument('CGTS') = 1) AND ^CGTS138 = NULL

        Data Elements:
            
                    RP:^FITR204 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS138 = CGTS:RP:gfagc.02.01:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                    */
        protected void VRATOGEN438025(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438025

            assertion = false; // = (FITR204 == @"D - Small business 15 year exemption" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438025",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business 15 year exemption amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR204", Value = FITR204 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438025

        #region VR.ATO.GEN.438027

        /*  VR.ATO.GEN.438027
                    If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed

                Legacy Rule Format:
                    IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [CGTS45] = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount

            Technical Business Rule Format:
                    (^FITR204 = 'F - Scrip for scrip roll-over') AND (CountDocument('CGTS') = 1) AND ^CGTS45 = NULL

        Data Elements:
            
                    RP:^FITR204 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            
                    RP:^CGTS45 = CGTS:RP:rvctc3.02.11:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                    */
        protected void VRATOGEN438027(List<ProcessMessageDocument> response, FITR2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438027

            assertion = false; // = (FITR204 == @"F - Scrip for scrip roll-over" && @"CGTS") == 1 &&  == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438027",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                    LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                    //Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } },
                };

                /*  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "FITR204", Value = FITR204 });


                */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438027
        #endregion Cross Form Rules - not supported yet
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(FITR2019 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOFITR434034(response, report);
            VRATOFITR434036(response, report);
            VRATOFITR434045(response, report);
            VRATOFITR434046(response, report);
            VRATOFITR434056(response, report);
            VRATOFITR434080(response, report);
            VRATOFITR434088(response, report);
           
            VRATOGEN438022(response, report);
            VRATOGEN438024(response, report);
           
            VRATOGEN438027(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
